<?php

namespace Drupal\evac\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\evac\ValidatorConfigTrait;
use Drupal\evac\ValidatorInfoTrait;

/**
 * The Email Validator Customizer Config Form.
 */
class ConfigForm extends ConfigFormBase {

  use ValidatorConfigTrait;
  use ValidatorInfoTrait;

  /**
   * Editable config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'evac_config_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [$this->configKey()];
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    parent::__construct($config_factory);
    $this->config = $this->config($this->configKey());
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $form = parent::buildForm($form, $form_state);

    $form['replace'] = [
      '#title' => $this->t('Replace the Email Validator service from core (RFCValidation).'),
      '#type' => 'checkbox',
      '#default_value' => $this->config->get('replace') ?? FALSE,
    ];

    $params = [
      ':url' => 'https://github.com/egulias/EmailValidator#available-validations',
    ];
    $replacements = $this->validationNames();
    // Remove the validation we intend to replace.
    unset($replacements['rfc']);
    $form['replacement'] = [
      '#title' => $this->t('Replacement validation'),
      '#type' => 'radios',
      '#description' => $this->t('Not sure which validation to choose? The <a href=":url" target="_blank" title=">egulias/EmailValidator">egulias/EmailValidator</a> documentation provides an overview of the various validation types.', $params),
      '#options' => $replacements,
      '#default_value' => $this->config->get('replacement') ?? array_key_first($replacements),
      '#states' => [
        'enabled' => [
          [':input[name="replace"]' => ['checked' => TRUE]],
        ],
      ],
    ];

    $params = [
      '@method' => $this->validationNames()['multiple_with_and'],
    ];
    $form['multiple_with_and'] = [
      '#title' => $this->t('Validations for @method', $params),
      '#type' => 'checkboxes',
      '#options' => $this->multipleWithAndOptions(),
      '#default_value' => $this->config->get('multiple_with_and') ?? [],
      '#states' => [
        'visible' => [
          [':input[name="replacement"]' => ['value' => 'multiple_with_and']],
        ],
        'enabled' => [
          [':input[name="replace"]' => ['checked' => TRUE]],
        ],

        // This required doesn't actually work as expected, so we have to check
        // in form validation.
        // @see https://www.drupal.org/project/drupal/issues/2950999
        'required' => [
          [':input[name="replacement"]' => ['value' => 'multiple_with_and']],
        ],
      ],
    ];

    $form['log_errors'] = [
      '#title' => $this->t('Log validation errors.'),
      '#description' => $this->t('Logs every validation error. Might generate a lot of records, depending in your case. Might be useful on production, but use with care.'),
      '#type' => 'checkbox',
      '#default_value' => $this->config->get('log_errors') ?? FALSE,
    ];

    $form['log_warnings'] = [
      '#title' => $this->t('Log validation warnings.'),
      '#type' => 'checkbox',
      '#description' => $this->t('Logs every validation warning. Might generate even more records. Not recommended for production.'),
      '#default_value' => $this->config->get('log_warnings') ?? FALSE,
      '#states' => [
        'enabled' => [
          [':input[name="log_errors"]' => ['checked' => TRUE]],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {

    $values = $form_state->getValues();
    if ($values['replace'] === 1 && $values['replacement'] === 'multiple_with_and') {

      $options = $values['multiple_with_and'];
      $options = array_flip($options);
      unset($options[0]);
      if (count($options) < 1) {

        $params = [
          '@method' => $this->validationNames()['multiple_with_and'],
        ];
        $message = $this->t('Choose at least 1 validation for @method to use.', $params);
        $form_state->setErrorByName('multiple_with_and', $message);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {

    $form_state->cleanValues();
    $values = $form_state->getValues();
    $this->config->setData($values)->save();

    parent::submitForm($form, $form_state);
  }

}
